@@ -4,6 +4,7 @@
 from __future__ import annotations
 
 import logging
+import secrets
 import subprocess
 from decimal import Decimal
 from unittest.mock import patch
@@ -13,10 +14,10 @@ import pytest
 from PIL import Image, UnidentifiedImageError
 
 from ocrmypdf._exec.ghostscript import DuplicateFilter, rasterize_pdf
-from ocrmypdf.exceptions import ExitCode
+from ocrmypdf.exceptions import ColorConversionNeededError, ExitCode
 from ocrmypdf.helpers import Resolution
 
-from .conftest import check_ocrmypdf, run_ocrmypdf
+from .conftest import check_ocrmypdf, run_ocrmypdf_api
 
 # pylint: disable=redefined-outer-name
 
@@ -29,8 +30,8 @@ def francais(resources):
 
 def test_rasterize_size(francais, outdir):
     path, pdf = francais
-    page_size_pts = (pdf.pages[0].MediaBox[2], pdf.pages[0].MediaBox[3])
-    assert pdf.pages[0].MediaBox[0] == pdf.pages[0].MediaBox[1] == 0
+    page_size_pts = (pdf.pages[0].mediabox[2], pdf.pages[0].mediabox[3])
+    assert pdf.pages[0].mediabox[0] == pdf.pages[0].mediabox[1] == 0
     page_size = (page_size_pts[0] / Decimal(72), page_size_pts[1] / Decimal(72))
     target_size = Decimal('50.0'), Decimal('30.0')
     forced_dpi = Resolution(42.0, 4242.0)
@@ -52,8 +53,8 @@ def test_rasterize_size(francais, outdir):
 
 def test_rasterize_rotated(francais, outdir, caplog):
     path, pdf = francais
-    page_size_pts = (pdf.pages[0].MediaBox[2], pdf.pages[0].MediaBox[3])
-    assert pdf.pages[0].MediaBox[0] == pdf.pages[0].MediaBox[1] == 0
+    page_size_pts = (pdf.pages[0].mediabox[2], pdf.pages[0].mediabox[3])
+    assert pdf.pages[0].mediabox[0] == pdf.pages[0].mediabox[1] == 0
     page_size = (page_size_pts[0] / Decimal(72), page_size_pts[1] / Decimal(72))
     target_size = Decimal('50.0'), Decimal('30.0')
     forced_dpi = Resolution(42.0, 4242.0)
@@ -75,8 +76,8 @@ def test_rasterize_rotated(francais, outdir, caplog):
         assert im.info['dpi'] == forced_dpi.flip_axis()
 
 
-def test_gs_render_failure(resources, outpdf):
-    p = run_ocrmypdf(
+def test_gs_render_failure(resources, outpdf, caplog):
+    exitcode = run_ocrmypdf_api(
         resources / 'blank.pdf',
         outpdf,
         '--plugin',
@@ -84,12 +85,12 @@ def test_gs_render_failure(resources, outpdf):
         '--plugin',
         'tests/plugins/gs_render_failure.py',
     )
-    assert 'Casper is not a friendly ghost' in p.stderr
-    assert p.returncode == ExitCode.child_process_error
+    assert 'TEST ERROR: gs_render_failure.py' in caplog.text
+    assert exitcode == ExitCode.child_process_error
 
 
-def test_gs_raster_failure(resources, outpdf):
-    p = run_ocrmypdf(
+def test_gs_raster_failure(resources, outpdf, caplog):
+    exitcode = run_ocrmypdf_api(
         resources / 'francais.pdf',
         outpdf,
         '--plugin',
@@ -97,12 +98,12 @@ def test_gs_raster_failure(resources, outpdf):
         '--plugin',
         'tests/plugins/gs_raster_failure.py',
     )
-    assert 'Ghost story archive not found' in p.stderr
-    assert p.returncode == ExitCode.child_process_error
+    assert 'TEST ERROR: gs_raster_failure.py' in caplog.text
+    assert exitcode == ExitCode.child_process_error
 
 
-def test_ghostscript_pdfa_failure(resources, outpdf):
-    p = run_ocrmypdf(
+def test_ghostscript_pdfa_failure(resources, outpdf, caplog):
+    exitcode = run_ocrmypdf_api(
         resources / 'francais.pdf',
         outpdf,
         '--plugin',
@@ -111,7 +112,7 @@ def test_ghostscript_pdfa_failure(resources, outpdf):
         'tests/plugins/gs_pdfa_failure.py',
     )
     assert (
-        p.returncode == ExitCode.pdfa_conversion_failed
+        exitcode == ExitCode.pdfa_conversion_failed
     ), "Unexpected return when PDF/A fails"
 
 
@@ -126,6 +127,16 @@ def test_ghostscript_feature_elision(resources, outpdf):
     )
 
 
+def test_ghostscript_mandatory_color_conversion(resources, outpdf):
+    with pytest.raises(ColorConversionNeededError):
+        check_ocrmypdf(
+            resources / 'jbig2_baddevicen.pdf',
+            outpdf,
+            '--plugin',
+            'tests/plugins/tesseract_noop.py',
+        )
+
+
 def test_rasterize_pdf_errors(resources, no_outpdf, caplog):
     with patch('ocrmypdf._exec.ghostscript.run') as mock:
         # ghostscript can produce
@@ -144,9 +155,10 @@ def test_rasterize_pdf_errors(resources, no_outpdf, caplog):
 
 
 class TestDuplicateFilter:
-    @pytest.fixture(scope='class', autouse=True)
+    @pytest.fixture(scope='function')
     def duplicate_filter_logger(self):
-        logger = logging.getLogger(__name__)
+        # token_urlsafe: ensure the logger has a unique name so tests are isolated
+        logger = logging.getLogger(__name__ + secrets.token_urlsafe(8))
         logger.setLevel(logging.DEBUG)
         logger.addFilter(DuplicateFilter(logger))
         return logger
@@ -162,9 +174,9 @@ class TestDuplicateFilter:
 
         assert len(caplog.records) == 5
         assert caplog.records[0].msg == "test error message"
-        assert caplog.records[1].msg == "(previous message repeated 2 times)"
+        assert caplog.records[1].msg == "(suppressed 2 repeated lines)"
         assert caplog.records[2].msg == "another error message"
-        assert caplog.records[3].msg == "(previous message repeated 1 times)"
+        assert caplog.records[3].msg == "(suppressed 1 repeated lines)"
         assert caplog.records[4].msg == "yet another error message"
 
     def test_filter_does_not_affect_unique_messages(
@@ -179,3 +191,20 @@ class TestDuplicateFilter:
         assert caplog.records[0].msg == "test error message"
         assert caplog.records[1].msg == "another error message"
         assert caplog.records[2].msg == "yet another error message"
+
+    def test_filter_alt_messages(self, duplicate_filter_logger, caplog):
+        log = duplicate_filter_logger
+        log.error("test error message")
+        log.error("another error message")
+        log.error("test error message")
+        log.error("another error message")
+        log.error("test error message")
+        log.error("test error message")
+        log.error("another error message")
+        log.error("yet another error message")
+
+        assert len(caplog.records) == 4
+        assert caplog.records[0].msg == "test error message"
+        assert caplog.records[1].msg == "another error message"
+        assert caplog.records[2].msg == "(suppressed 5 repeated lines)"
+        assert caplog.records[3].msg == "yet another error message"
